import c from 'chalk';
import inquirer from 'inquirer';
import * as path from 'node:path';
import * as fs from '../utils/fs.js';
import { templates, Template } from '../helpers/templates.js';
// Constants
const __cwd = process.cwd();
const help = [
    'Usage:',
    `  nodelink init ${c.green('(path)')} ${c.yellow('{args}')}`,
    '',
    `${c.yellow('Arguments')}:`,
    `  ${c.yellow('--help/-h')}: Shows the help this ${c.cyan('command')}.`
].join('\n');
export async function initialize() {
    const basename = path.basename(__cwd);
    // Project Type
    const types = new Map();
    for (const template of templates) {
        let ref = types;
        for (let i = 0; i < template.type.length; i++) {
            const type = template.type[i];
            if (i === template.type.length - 1) {
                ref.set(type, template);
            }
            else {
                let typeExists = false;
                ref.forEach((value, key) => {
                    if (Template.isTemplateType(key) && key.name === type.name) {
                        typeExists = true;
                        ref = value;
                    }
                });
                if (!typeExists) {
                    ref.set(type, new Map());
                    ref = ref.get(type);
                }
            }
        }
    }
    // Project Name
    const name = (await inquirer.prompt([{
            type: 'text',
            name: 'name',
            message: 'What is the project name?',
            default: basename
        }])).name;
    const projectPath = path.resolve(__cwd, `${name === basename ? '..' : '.'}/${name}`);
    const isEmpty = fs.isEmptyDirectory(projectPath);
    if (!isEmpty) {
        const override = (await inquirer.prompt([{
                type: 'confirm',
                name: 'override',
                message: c.yellow('Project directory is not empty, do you want to continue?'),
                default: false
            }])).override;
        if (!override) {
            console.error(c.red('Operation canceled.'));
            return;
        }
    }
    let template = null;
    let choices = types;
    while (!template) {
        const type = (await inquirer.prompt([{
                type: 'list',
                name: 'type',
                message: 'What kind of project are you building?',
                default: choices.values().next(),
                choices: Array.from(choices.entries()).map((e) => {
                    return {
                        name: c.hex(e[0].color)(e[0].name),
                        value: e[0]
                    };
                })
            }])).type;
        choices = choices.get(type);
        if (choices instanceof Template) {
            template = choices;
        }
    }
    // Initialization
    fs.ensureDirSync(projectPath);
    await template.clone(name, projectPath);
    console.info(`${c.green('Project was created with success:')} ${projectPath}`);
}
export default async (argv) => {
    if (argv.h || argv.help) {
        console.info(help);
    }
    else {
        await initialize();
    }
};
