#!/usr/bin/env node
// Helpers
import yargs from 'yargs';
import { hideBin } from 'yargs/helpers';
import * as fs from 'node:fs';
import * as url from 'node:url';
import * as path from 'node:path';
import c from 'chalk';
// Constants
const __dirname = path.dirname(url.fileURLToPath(import.meta.url));
const { version } = JSON.parse(fs.readFileSync(path.resolve(__dirname, '../package.json'), 'utf8'));
const help = [
    `${c.green(`Nodelink v${version} in use.`)}`,
    '',
    'Usage:',
    `  nodelink ${c.cyan('[cmd]')} ${c.yellow('{args}')}`,
    '',
    `${c.cyan('Commands')}:`,
    `  ${c.cyan('help')}: Show those helpful notes.`,
    `  ${c.cyan('version')}: Shows Nodelink version in use.`,
    `  ${c.cyan('init')}: Initializes a project.`,
    '',
    `${c.yellow('Arguments')}:`,
    `  ${c.yellow('--help/-h')}: Shows the help for a ${c.cyan('command')}.`,
    `  ${c.yellow('--version/-v')}: Shows Nodelink version in use.`
].join('\n');
(async () => {
    const argv = await yargs(hideBin(process.argv)).help(false).argv;
    if (argv._.length === 0) {
        if (argv.v || argv.version) {
            console.info(version);
        }
        else {
            console.info(help);
        }
    }
    else if (argv._.length === 1 && argv._[0] === 'help') {
        console.info(help);
    }
    else if (argv._.length === 1 && argv._[0] === 'version') {
        console.info(version);
    }
    else {
        // Command
        let cmd;
        try {
            cmd = await import(`./api/${argv._[0]}.js`);
        }
        catch {
            console.error(c.red(`Command ${c.yellow(argv._[0].toString())} does not exist.`));
            process.exit(1);
        }
        try {
            await cmd.default(argv);
        }
        catch (err) {
            console.error('An error has occured:');
            console.error(err);
            process.exit(1);
        }
    }
})();
