// Helpers
import * as path from 'node:path';
import * as url from 'node:url';
import c from 'chalk';
import * as Eta from 'eta';
import * as fs from '../utils/fs.js';
import * as types from '../utils/types.js';
// Constants
const __dirname = path.dirname(url.fileURLToPath(import.meta.url));
const templatePath = path.resolve(__dirname, '../templates');
export const templates = [];
export class Template {
    constructor(manifest, dir) {
        if (!Template.isTemplateManifest(manifest)) {
            throw new Error('Invalid manifest syntax.');
        }
        this.path = path.dirname(dir);
        this.type = manifest.type;
    }
    // Methods
    async clone(name, dest) {
        console.info(c.cyan(`Cloning ${c.green(this.type[this.type.length - 1].name)} template...`));
        const context = {
            name
        };
        let currentPath = this.path;
        while (currentPath !== templatePath) {
            const commonPath = path.resolve(currentPath, '../_common');
            if (fs.existsSync(commonPath)) {
                for (const file of fs.readGlobSync(commonPath)) {
                    const location = path.relative(commonPath, file);
                    const content = await Eta.renderFile(file, context, { autoTrim: false });
                    fs.writeFileSync(path.resolve(dest, location), content);
                }
            }
            currentPath = path.resolve(currentPath, '../');
        }
        for (const file of fs.readGlobSync(this.path)) {
            if (file !== path.resolve(this.path, 'metadata.json')) {
                const location = path.relative(this.path, file);
                const content = await Eta.renderFile(file, context, { autoTrim: false });
                fs.writeFileSync(path.resolve(dest, location), content);
            }
        }
    }
    // Static
    static isTemplateType(type) {
        return (type &&
            // Name
            types.isString(type.name) &&
            // Color
            types.isString(type.color));
    }
    static isTemplateTypeArray(arr) {
        if (Array.isArray(arr)) {
            arr.forEach((item) => {
                if (!Template.isTemplateType(item)) {
                    return false;
                }
            });
            return true;
        }
        else {
            return false;
        }
    }
    static isTemplateManifest(manifest) {
        return (manifest &&
            // Type
            Template.isTemplateTypeArray(manifest.type));
    }
}
for (const file of fs.readGlobSync(templatePath)) {
    if (path.basename(file) === 'metadata.json') {
        try {
            const manifest = JSON.parse(fs.readFileSync(file, 'utf8'));
            templates.push(new Template(manifest, file));
        }
        catch (err) {
            const relative = path.relative(templatePath, file);
            console.warn(`"${relative}": ${c.yellow(err)}`);
        }
    }
}
